<?php
/**
 * FID Acquisition Trait
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2010.
 * Copyright (C) Leipzig University Library 2015.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind FID
 * @package  Controller
 * @author   Alexander Purr <purr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 */
namespace fid\Controller\CustomTraits;

use fid\Service\Client;
use fid\Service\DataTransferObject\Order;
use fid\Service\DataTransferObject\User;
use fid\Validator\SubitoPartialCopyPageBounds;
use fid\Validator\SubitoPartialCopyPageSelection;
use finc\View\Helper\Root\Citation;
use VuFind\Exception\Forbidden as ForbiddenException;
use Zend\Form\Form;

/**
 * FID Acquisition Trait
 *
 * @category VuFind
 * @package  Controller
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     http://finc.info   Main Site
 */
trait FidAcquisitionTrait
{
    /**
     * @var Client
     */
    protected $client;

    protected $type;
    protected $formConfig = "fid-acquisition-form";

    /**
     * @param Client $client
     */
    public function setClient(Client $client): void
    {
        $this->client = $client;
    }

    public function fidPdaAction()
    {
        $this->type = self::PDA;
        return $this->runAcquisition();
    }

    public function fidSubitoArticleAction()
    {
        $this->type = self::SUBITO_ARTICLE;
        return $this->runAcquisition();
    }

    public function fidSubitoPartialCopyAction()
    {
        $this->type = self::SUBITO_PARTIAL_COPY;
        $this->formConfig = ('fid-acquisition-subito-partial-copy-form');
        return $this->runAcquisition();
    }

    protected function runAcquisition()
    {
        if (!($user = $this->getUser())) {
            return $this->forceLogin();
        }

        /** @var User $user */
        $user = $this->client->requestUserDetails();

        try {
            /* pass if permission is granted, else throw exception and switch to catch block */
            $this->permission()->check('fid.Acquisitions', 'exception');

            $driver = $this->loadRecord();
            $recordId = $driver->tryMethod('getUniqueID');

            /** @var Form $form */
            $form = $this->serviceLocator->get($this->formConfig);

            $this->addOptinalFormValidators($form);

            if ($this->formWasSubmitted()) {
                $form->setData($this->getRequest()->getPost());
                if ($form->isValid()) {
                    $url = $this->getRecordUrl($recordId);
                    $label = $this->getOrderLabel($driver);
                    $type = $this->type;
                    $hydratorData = compact('type', 'driver', 'user', 'url', 'label');
                    $hydratorData += $this->addTypeSpecificOrderInformation($form);
                    $form->getHydrator()->hydrate($hydratorData, $order = new Order());
                    $this->client->requestOrderCreation($order);
                    $this->client->requestUserDetails(null, true);
                    $message = $this->translate('fid::acquisition_success');
                    $messenger = $this->flashMessenger();
                    $messenger->addSuccessMessage($message);
                    return $this->redirect()->toRoute('record', ['id' => $recordId]);
                } else {
                    $this->handleOptionalErrorMessages($form);
                }
            }

            $typeWithoutHyphens = preg_replace("/[^a-zA-Z]+/", "", $this->type);
            $action = $this->url()->fromRoute("record-fid$typeWithoutHyphens", ['id' => $recordId]);
            $form->setAttribute('action', $action);
            $form->prepare();

            $view = $this->createViewModel();
            $view->setVariables(compact('form', 'driver', 'user'));
            $view->setTemplate("fid/record/acquisition-$this->type");
        } catch (ForbiddenException $ex) {
            $view = $this->getPermissionDeniedView();
        }

        return $view;
    }

    protected function getRecordUrl($recordId)
    {
        return $this->url()->fromRoute('record', ['id' => $recordId], ['force_canonical' => true]);
    }

    protected function getRecordCitation()
    {
        $driver = $this->loadRecord();
        /** @var Citation $helper */
        $helper = $this->getViewRenderer()->plugin('citation');
        $helper->__invoke($driver);
        $priorityFormats = [ 'APA', 'MLA', 'ISBD' ];
        $existingFormats = $driver->getCitationFormats();
        $formats = array_intersect($priorityFormats, $existingFormats);
        if (!empty($formats)) {
            return $helper->getCitation($formats[0]);
        } else {
            return $driver->tryMethod('getTitle');
        }
    }

    protected function getOrderLabel()
    {
        return $this->loadRecord()->tryMethod('getTitle');
    }

    protected function getPermissionDeniedView()
    {
        $view = $this->createViewModel();
        $view->setVariables(['msg' => "fid::acquisition_permission_denied"]);
        $view->setTemplate('error/permissiondenied.phtml');
        return $view;
    }

    protected function addOptinalFormValidators($form)
    {
        if ($this->type == self::SUBITO_PARTIAL_COPY) {
            // add number of pages specific validators
            $recordPages = $this->loadRecord()->tryMethod('getNumberOfPages');
            if ($recordPages) {
                $form->getInputFilter()->get('pages')->getValidatorChain()->attach(
                    new SubitoPartialCopyPageBounds(['numberPages' => $recordPages]),
                    true
                );
                $form->getInputFilter()->get('pages')->getValidatorChain()->attach(
                    new SubitoPartialCopyPageSelection(['numberPages' => $recordPages])
                );
            }
        }
    }

    protected function addTypeSpecificOrderInformation($form)
    {
        $data = [];
        if ($this->type == self::SUBITO_PARTIAL_COPY) {
            $data += [
                'subitoPartialCopy' => [
                    'selection' => $form->getData()['pages'],
                    'comment' => $form->getData()['comment']
                ]
            ];
        }
        return $data;
    }

    protected function handleOptionalErrorMessages($form)
    {
    }
}
